# -*- coding: utf-8 -*-
#
# Created on Apr 13, 2016
# @authors: Jiri Spilka
# http://people.ciirc.cvut.cz/~spilkjir
# @2016, CIIRC, Czech Technical University in Prague
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see CTGViewerLite.py for details)

import PyQt4.Qwt5 as Qwt
from PyQt4 import QtGui, Qt, QtCore

import numpy as np
import itertools
import textwrap


class EnumAnnType:
    """
    Types of annotations: basal, baseline, recovery
    """
    def __init__(self):
        pass

    select = 'select'
    basal = 'basal'
    baseline = 'baseline'
    recovery = 'recovery'
    no_recovery = 'no_recovery'
    excessive_ua = 'excessive_ua'
    ellipse = 'ellipse'
    ellipsenote = 'ellipsenote'
    note = 'note'


class AnnotationObject(object):

    newid = itertools.count().next

    def __init__(self, parent_name='', curve_type=None, x1=None, x2=None, y1=None, y2=None, s=''):

        self.id = AnnotationObject.newid()

        # information to be saved in annotation file
        self.__parent_name = parent_name
        self.__curve_type = curve_type
        self.x_from = x1
        self.x_to = x2
        self.yval1 = y1
        self.yval2 = y2
        self.valid = True  # if a curve is valid and should be plotted
        self.text = Qwt.QwtText(self.text_wrap(str(s)))

        self.font_app = QtGui.QFont()

        # QPen settings
        self.pen_def = Qt.QPen()
        self.pen_symbol = Qt.QPen()
        self.symbol = Qwt.QwtSymbol()

        self.__default_pen = Qt.QPen()
        self.color_highlight = Qt.Qt.darkMagenta

    def get_default_pen(self):
        return self.__default_pen

    def set_default_pen(self):
        self.__default_pen = Qt.QPen(self.pen_def)

    def get_symbol(self):
        return self.symbol

    def get_symbol_highlight(self):
        symbol = Qwt.QwtSymbol(self.symbol)
        pen = Qt.QPen(self.pen_symbol)
        pen.setColor(self.color_highlight)
        symbol.setPen(pen)
        return symbol

    def get_color_highlight(self):
        return self.color_highlight

    def get_curve_type(self):
        return self.__curve_type

    def get_parent_name(self):
        return self.__parent_name

    def define_pen(self, c=Qt.Qt.gray, w=4, l=Qt.Qt.SolidLine):
        self.pen_def = Qt.QPen(c)
        self.pen_def.setWidth(w)
        self.pen_def.setStyle(l)

    def define_pen_symbol(self, c=Qt.Qt.black, w=4, l=Qt.Qt.SolidLine):
        self.pen_symbol = Qt.QPen(c)
        self.pen_symbol.setWidth(w)
        self.pen_symbol.setStyle(l)

    def define_symbol(self, s=Qwt.QwtSymbol.VLine, c=Qt.Qt.black, qsize=Qt.QSize(15, 15)):
        self.symbol = Qwt.QwtSymbol(s, Qt.QBrush(c), self.pen_symbol, qsize)

    def define_pen_symbol_light(self):
        c = self.pen_def.color()
        c.setAlpha(120)
        self.pen_def.setColor(c)
        self.pen_def.setWidth(3)
        self.pen_symbol.setWidth(3)
        self.pen_symbol.setColor(c)

        self.define_symbol(Qwt.QwtSymbol.VLine, c, Qt.QSize(7, 7))

    def set_xy_values(self, x1, x2, y1=None, y2=None):
        self.x_from = x1
        self.x_to = x2
        self.yval1 = y1
        self.yval2 = y2

    def correct_val_order(self):

        if self.x_from > self.x_to:
            self.x_from, self.x_to = self.x_to, self.x_from

        if self.yval1 > self.yval2:
            self.yval1, self.yval2 = self.yval2, self.yval1

    def is_too_small(self):
        return False

    def text_wrap(self, s):
        w = 140
        s = s.replace('\n', ' ')
        s = textwrap.fill(s, w / 8 + 1)
        return s

    def get_text(self):
        return str(self.text.text())

    def set_text(self, s):
        self.text.setText(s)

    def set_text_property(self):
        c = Qt.QColor(Qt.Qt.black)
        self.text.setColor(c)
        c.setAlpha(20)
        self.text.setBackgroundBrush(c)
        self.font_app.setBold(False)
        self.text.setFont(self.font_app)


class PyQwtPlotCurveAnnotator(AnnotationObject, Qwt.QwtPlotCurve):
    """
    Definitions for basal heart rate, baseline, recovery, and no recovery
    """

    def __init__(self, parent_name='', curve_type=None, x1=None, x2=None, y1=None, y2=None, s=''):

        super(AnnotationObject, self).__init__()
        AnnotationObject.__init__(self, parent_name, curve_type, x1, x2, y1, y2, s)
        Qwt.QwtPlotCurve.__init__(self)

        if curve_type == EnumAnnType.basal:
            c = Qt.QColor(Qt.Qt.gray)
            c.setAlpha(200)
            self.define_pen(c, 4)
            c.setAlpha(0)
            self.define_pen_symbol(c)
            self.define_symbol()

        elif curve_type == EnumAnnType.baseline:
            self.define_pen(Qt.Qt.blue, 4, Qt.Qt.DotLine)
            self.define_pen_symbol(Qt.Qt.blue)
            self.define_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.blue)

        elif curve_type == EnumAnnType.recovery:
            self.define_pen(Qt.Qt.green)
            self.define_pen_symbol(Qt.Qt.green)
            self.define_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.green, Qt.QSize(15, 15))

        elif curve_type == EnumAnnType.no_recovery:
            self.define_pen(Qt.Qt.red)
            self.define_pen_symbol(Qt.Qt.red)
            self.define_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.red, Qt.QSize(15, 15))

        elif curve_type == EnumAnnType.excessive_ua:
            self.define_pen(Qt.Qt.red, 4, Qt.Qt.DotLine)
            self.define_pen_symbol(Qt.Qt.red)
            self.define_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.red, Qt.QSize(15, 15))

        else:
            raise Exception('Do not use PyQwtPlotCurveAnnotator for this type of curve')

        self.set_default_pen()
        self.setPen(self.pen_def)
        self.setSymbol(self.symbol)

    def copy(self):
        pn = self.get_parent_name()
        t = self.get_curve_type()
        return PyQwtPlotCurveAnnotator(pn, t, self.x_from, self.x_to, self.yval1, self.yval2, self.get_text())

    def is_too_small(self):
        return abs(self.x_to - self.x_from) < 1

    def correct_val_order(self):

        if self.x_from > self.x_to:
            self.x_from, self.x_to = self.x_to, self.x_from

    def set_pen_symbol_light(self):
        self.define_pen_symbol_light()
        self.setPen(self.pen_def)
        self.setSymbol(self.symbol)

    def plot(self):
        x = [self.x_from, self.x_to]
        y = self.yval1 * np.ones((len(x), 1), float)

        self.setData(x, y)

    def plot_xy(self, x1, x2, y1, y2=None):
        x = [x1, x2]
        y1 = y1 * np.ones((len(x), 1), float)

        self.setData(x, y1)

    def draw_my(self, x, y):
        self.draw(x, y)


class PyQwtPlotMarkerAnnotator(AnnotationObject, Qwt.QwtPlotMarker):
    """
    Definition of annotation box. This class is base class for basal, baseline, recovery index, note etc.
    """

    def __init__(self, parent_name='', curve_type=None, x1=None, x2=None, y1=None, y2=None, s=''):
        super(AnnotationObject, self).__init__()
        AnnotationObject.__init__(self, parent_name, curve_type, x1, x2, y1, y2, s)
        Qwt.QwtPlotMarker.__init__(self)

        if curve_type == EnumAnnType.note:

            self.setLineStyle(Qwt.QwtPlotMarker.VLine)
            self.setLabelAlignment(Qt.Qt.AlignRight | Qt.Qt.AlignTop)

            c = Qt.QColor(Qt.Qt.black)
            c.setAlpha(30)
            self.define_pen(c, 3)
            self.setLinePen(self.pen_def)

            self.set_text_property()

            # c = Qt.QColor(Qt.Qt.red)
            # c.setAlpha(50)
            # self.define_pen_symbol(c)
            # self.define_symbol()
        else:
            raise Exception('Do not use PyQwtPlotMarkerAnnotator for this type of curve')

        self.set_default_pen()
        self.setPen(self.pen_def)
        self.setSymbol(self.symbol)

    def copy(self):
        pn = self.get_parent_name()
        t = self.get_curve_type()
        return PyQwtPlotMarkerAnnotator(pn, t, self.x_from, self.x_to, self.yval1, self.yval2, self.get_text())

    def define_pen_symbol_light(self):
        c = self.pen_def.color()
        c.setAlpha(60)
        self.pen_def.setColor(c)
        self.pen_def.setWidth(2)
        self.define_symbol(Qwt.QwtSymbol.VLine, c, Qt.QSize(7, 7))

        f = QtGui.QFont(self.font_app)
        f.setPointSize(self.font_app.pointSize() - 2)
        self.text.setFont(f)

    def set_pen_symbol_light(self):
        self.define_pen_symbol_light()
        self.setPen(self.pen_def)

    def correct_val_order(self):
        """ Not used """
        pass

    def plot(self):
        self.setLabel(self.text)
        self.setValue(self.x_from, 0)

    def plot_xy(self, x1, x2=None, y1=None, y2=None):
        self.setValue(x1, 0)

    def setPen(self, pen):
        self.setLinePen(pen)

    def draw_my(self, x=None, y=None):
        """auxially function"""
        self.plot()


class PyQwtPlotEllipseAnnotator(AnnotationObject, Qwt.QwtPlotCurve):
    """
    Definitions for an ellipse
    """
    newid = itertools.count().next

    def __init__(self, parent_name='', curve_type=None, x1=None, x2=None, y1=None, y2=None, s=''):
        super(AnnotationObject, self).__init__()
        AnnotationObject.__init__(self, parent_name, curve_type, x1, x2, y1, y2, s)
        Qwt.QwtPlotCurve.__init__(self)

        if curve_type == EnumAnnType.ellipse or curve_type == EnumAnnType.ellipsenote:
            self.define_pen(Qt.Qt.blue, 3, Qt.Qt.SolidLine)
            self.define_pen_symbol(Qt.Qt.blue)
            self.define_symbol(Qwt.QwtSymbol.VLine, Qt.Qt.blue)

            # define fill of ellipse
            c = Qt.QColor(Qt.Qt.blue)
            c.setAlpha(20)
            self.brush = Qt.QBrush(c)
            self.setBrush(Qt.QBrush(c))

            # define pen for horizontal line
            c = Qt.QColor(Qt.Qt.black)
            c.setAlpha(40)
            self.pen_hline = Qt.QPen(c)
            self.pen_hline.setWidth(2)
            self.pen_hline.setStyle(Qt.Qt.SolidLine)

            self.set_text_property()

        else:
            raise Exception('Do not use PyQwtPlotEllipseAnnotator for this type of curve')

        self.set_default_pen()
        self.setPen(self.pen_def)
        self.setSymbol(self.symbol)

    def copy(self):
        pn = self.get_parent_name()
        t = self.get_curve_type()
        return PyQwtPlotEllipseAnnotator(pn, t, self.x_from, self.x_to, self.yval1, self.yval2, self.get_text())

    def draw(self, painter, xMap, yMap, canvasRect):
        """
        :param painter:
        :type painter: QtGui.QPainter
        """

        painter.setPen(self.pen())
        painter.setBrush(self.brush)

        # print canvasRect.left(), canvasRect.right(), canvasRect.bottom(), canvasRect.top()
        # print self.x(0), self.x(1), self.y(0), self.y(1)
        # print xMap.transform(self.x(0)), xMap.transform(self.x(1)), yMap.transform(self.y(0)), yMap.transform(self.y(1))

        i = 0
        px1 = xMap.transform(self.x(i))
        py1 = yMap.transform(self.y(i))
        px2 = xMap.transform(self.x(i + 1))
        py2 = yMap.transform(self.y(i + 1))

        # w = py2 - py1  # width of rectangle
        # ybottom = py1 + w

        px2 = px2 if px2 > canvasRect.left() else canvasRect.left()-2
        py2 = py2 if py2 < canvasRect.bottom() else canvasRect.bottom()
        py2 = py2 if py2 > canvasRect.top() else canvasRect.top()

        # print px1, px2, py1, py2
        # print py2 - py1

        # drawEllipse(int x, int y, int width, int height)
        painter.drawEllipse(px1, py1, px2 - px1, py2 - py1)

        if self.get_curve_type() == EnumAnnType.ellipsenote and len(self.get_text()) > 0:

            x_line = min(px1, px2-1)

            painter.setPen(self.pen_hline)
            painter.drawLine(x_line, py1 + (py2 - py1)/2, x_line, canvasRect.top())

            textsize = self.text.textSize(painter.font())
            texrect = QtCore.QRect(x_line, canvasRect.top(), textsize.width(), textsize.height())

            self.text.draw(painter, texrect)

    def set_pen_symbol_light(self):
        self.define_pen_symbol_light()
        self.setPen(self.pen_def)

    def is_too_small(self):
        return abs(self.x_to - self.x_from) < 100

    # def correct_val_order(self):
    #     pass

    def plot(self):
        self.plot_xy(self.x_from, self.x_to, self.yval1, self.yval2)

    def plot_xy(self, x1, x2, y1, y2):

        x = [x1, x2]
        y = [y1, y2]

        self.setData(x, y)

    def draw_my(self, x=None, y=None):
        """auxially function"""
        self.plot()