# -*- coding: utf-8 -*-
#
# Created on Nov 27, 2015
# @authors: Jiri Spilka
# http://people.ciirc.cvut.cz/~spilkjir
# @2015, CIIRC, Czech Technical University in Prague
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see CTGViewerLite.py for details)

"""
Annotator
---------
Some text

Reference
~~~~~~~~~

.. autoclass:: Annotator
    :members:
    :private-members:

"""

from PyQt4 import Qt
from PyQt4.QtCore import pyqtSignal, QPointF
import PyQt4.Qwt5 as Qwt
import numpy as np
import os
import logging

import Common
from GuiForms import AddNoteDialog
from Config import ConfigStatic
from AnnotationObject import EnumAnnType, PyQwtPlotCurveAnnotator, \
    PyQwtPlotMarkerAnnotator, PyQwtPlotEllipseAnnotator, AnnotationObject


class Annotator:
    """
    Class Annotator manages loading and saving of annotations stored in an .ann file.
    """

    def __init__(self):

        self._ann_extension = '.ann'
        self._ann_file = ''
        self._path = ''
        self._dannotations_fhr = dict()
        self._dannotations_toco = dict()

        self._sep_items = ','
        self._sep_anns = '\n'

        self._log = logging.getLogger(ConfigStatic.logger_name)

    def __load_annotations(self):
        """
        Load annotations from .ann file.

        :return: annotations in dict()
        :rtype: dict()
        """

        self._dannotations_fhr = dict()
        self._dannotations_toco = dict()

        if self.get_signal_annotated():
            self._log.info("Loading annotation file: {0}".format(self._ann_file))
            with open(self._ann_file, 'r') as fp:
                for line in fp.readlines():
                    # print line.split(',')

                    s = line.split(',')

                    if len(s) != 8:
                        if len(s) == 5:
                            self._log.warning("Annotations in OLD format!! {0}".format(s))
                            continue
                        else:
                            self._log.warning("Annotations in wrong format!! {0}".format(s))

                    id_curve = str(s[0])
                    parent_name = str(s[1])
                    curve_type = str(s[2])
                    x_from = int(s[3])
                    x_to = int(s[4])
                    yval1 = self.check_int(s[5])
                    yval2 = self.check_int(s[6])
                    note = self.check_str(s[7])

                    if not(curve_type == EnumAnnType.basal or curve_type == EnumAnnType.baseline or
                            curve_type == EnumAnnType.recovery or curve_type == EnumAnnType.no_recovery or
                            curve_type == EnumAnnType.excessive_ua or curve_type == EnumAnnType.note or
                            curve_type == EnumAnnType.ellipse or EnumAnnType.ellipsenote):
                        raise Exception('Unsuported annotation type')

                    if curve_type == EnumAnnType.note:
                        marker = PyQwtPlotMarkerAnnotator(parent_name, curve_type, x_from, x_to, None, None, note)

                    elif curve_type == EnumAnnType.ellipse or curve_type == EnumAnnType.ellipsenote:
                        marker = PyQwtPlotEllipseAnnotator(parent_name, curve_type, x_from, x_to, yval1, yval2, note)

                    else:
                        marker = PyQwtPlotCurveAnnotator(parent_name, curve_type, x_from, x_to, yval1, yval2)

                    marker.id = id_curve

                    if parent_name.lower() == 'toco':
                        self._dannotations_toco[marker.id] = marker
                    else:
                        self._dannotations_fhr[marker.id] = marker

        else:
            self._dannotations_fhr = dict()
            self._dannotations_toco = dict()

        # return self._dannotations_fhr

    def check_int(self, d):
        if d == 'None':
            return None
        else:
            return int(d)

    def check_str(self, s):
        if s == 'None' or s == '\n':
            return ''
        else:
            return str(s)

    def ann_file_load(self, data_file):
        """
        Set the annotation file and load it.

        :param data_file: annotation file
        :type data_file: str
        :return:
        """
        name = Common.get_filename_without_ext(data_file)

        self._path = os.path.dirname(data_file)
        # set annotations
        self._ann_file = os.path.join(self._path, name + self._ann_extension)

        # load annotations
        self.__load_annotations()
        # return self._dannotations_fhr

    def get_signal_annotated(self):
        b = Common.file_exists(self._ann_file) and Common.get_mumber_lines(self._ann_file) > 0
        return b

    def get_annotations_fhr(self):
        return self._dannotations_fhr

    def get_annotations_toco(self):
        return self._dannotations_toco

    def get_annotations_copy_all(self):

        # for key, curve in self._dannotations_fhr.iteritems():
        #     print key, curve.get_parent_name()

        z = self._dannotations_fhr.copy()
        z.update(self._dannotations_toco)

        # for key, curve in z.iteritems():
        #     print key, curve.get_parent_name()
        return z

    # def get_annotations_copy(self):
    #     self._dannotations_fhr.copy()

    def get_annotations_copy(self):

        ann = dict()
        for key, curve in self._dannotations_fhr.iteritems():
            name = curve.get_parent_name()
            typ = curve.get_curve_type()
            if typ == EnumAnnType.note:
                c = PyQwtPlotMarkerAnnotator(name, typ, curve.x_from, curve.x_to, curve.yval1, None,
                                             curve.get_text())

            elif typ == EnumAnnType.ellipse or typ == EnumAnnType.ellipsenote:
                c = PyQwtPlotEllipseAnnotator(name, typ, curve.x_from, curve.x_to, curve.yval1, curve.yval2,
                                              curve.get_text())
            else:
                c = PyQwtPlotCurveAnnotator(name, typ, curve.x_from, curve.x_to, curve.yval1)

            ann[curve.id] = c

        return ann

    def set_annotations(self, dann_fhr, dann_toco):
        self._dannotations_fhr = dann_fhr
        self._dannotations_toco = dann_toco

    def save_annotations(self):

        z = self._dannotations_fhr.copy()
        z.update(self._dannotations_toco)

        with open(self._ann_file, 'w+') as fp:
            for idd, d in z.items():

                name = str(d.get_parent_name())
                curve_type = str(d.get_curve_type())

                if d.valid:
                    fp.write(str(idd))
                    fp.write(self._sep_items)
                    fp.write(name)
                    fp.write(self._sep_items)
                    fp.write(curve_type)
                    fp.write(self._sep_items)
                    fp.write(str(d.x_from))
                    fp.write(self._sep_items)
                    fp.write(str(d.x_to))
                    fp.write(self._sep_items)
                    fp.write(str(d.yval1))
                    fp.write(self._sep_items)
                    fp.write(str(d.yval2))
                    fp.write(self._sep_items)

                    s = d.get_text()
                    s = s.replace('\n', ' ')
                    fp.write(s)
                    fp.write(self._sep_anns)

        self._log.info("Saving annotation file: {0}".format(self._ann_file))

    def set_annotations_and_save(self, dann_fhr, dann_toco):
        self.set_annotations(dann_fhr, dann_toco)
        self.save_annotations()


class CanvasPickerAnnotator(Qt.QObject):
    """
    Pick events generated by user such as draw basal, baseline, recovery, etc.
    The CanvasPickerAnnotator calls super function to plot the events

    :param plot: x axis values
    :type plot: FhrPlot
    """

    signal_ann_moved = pyqtSignal()
    signal_ann_moved_outside_view = pyqtSignal(['int'])

    def __init__(self, p_plot):
        Qt.QObject.__init__(self, p_plot)

        self.__selected_curve = None
        self.__selected_point = -1
        self.__point_clicked = None

        self.__plot = p_plot

        self._add_note_dialog = AddNoteDialog()

        self.canvas = self.__plot.canvas()
        self.canvas.installEventFilter(self)

        # # We want the focus, but no focus rect.
        # # The selected point will be highlighted instead.
        self.canvas.setFocusPolicy(Qt.Qt.StrongFocus)
        # canvas.setCursor(Qt.Qt.PointingHandCursor)
        self.canvas.setFocusIndicator(Qwt.QwtPlotCanvas.ItemFocusIndicator)
        self.canvas.setFocus()

        # self._distance_point = 100
        # self._distance_line = 5
        # self._distance_note = 300
        # self._distance_ellipse = 200
        self._distance = 0.02
        # self._distance_ellipse_center = 500

        self.__bmoving = False
        self.__addannotation = False
        # self._current_curve = AnnotationObject()
        # self.__ann_action = self.__plot.get_ann_action()

        self.__curve_to_copy = None
        self.__last_click_pos = None

        # self.canvas.setMouseTracking(True)
        # self.__shiftCurveCursor(True)

    def compute_dist_ellipse(self, x_from, x_to, y1, y2, pos):
        centerx = x_from + (x_to - x_from) / 2
        centery = y1 + (y2 - y1) / 2

        # print centerx, x_from, x_to
        # print centery, y1, y2

        dcxcy = self.distance_to_point(pos, centerx, centery)
        dcxy1 = self.distance_to_point(pos, centerx, y1)
        dcxy2 = self.distance_to_point(pos, centerx, y2)
        dx1cy = self.distance_to_point(pos, x_from, centery)
        dx2cy = self.distance_to_point(pos, x_to, centery)

        # print dcxcy, dcxy2, dx2cy, dcxy1, dx1cy
        return dcxcy, dcxy2, dx2cy, dcxy1, dx1cy

    def distance_to_point(self, pos, x, y):
        dx = pos.x() - x
        dy = pos.y() - y
        return np.sqrt(dx ** 2 + dy ** 2)

    def __compute_point(self, pos):
        """
        The clicked point stored in event.x() and event.y() is transformed to x and y axis for
        particular plot attached.

        :param pos: x and y position
        :type  pos: QPointF
        :rtype: QPointF
        """
        clickedpointx = self.__plot.invTransform(Qwt.QwtPlot.xBottom, pos.x())
        clickedpointy = self.__plot.invTransform(Qwt.QwtPlot.yLeft, pos.y())

        return Qt.QPoint(int(clickedpointx), int(clickedpointy))

    def __check_outside_of_plot(self, x1, x2, y1, y2):

        xmin = self.__plot.xAxisMinSample()
        xmax = self.__plot.xAxisMaxSample()

        ymin = self.__plot.viewYMinSample()
        ymax = self.__plot.viewYMaxSample()

        x1 = x1 if x1 > xmin else xmin
        x1 = x1 if x1 < xmax else xmax

        x2 = x2 if x2 > xmin else xmin
        x2 = x2 if x2 < xmax else xmax

        y1 = y1 if y1 > ymin else ymin
        y1 = y1 if y1 < ymax else ymax

        y2 = y2 if y2 > ymin else ymin
        y2 = y2 if y2 < ymax else ymax

        return x1, x2, y1, y2

    def __select(self, pos, bhighlight=True, select_type=None):
        """

        :param pos:
        :param bhighlight:
        :param select_type:
        :type pos: QPointF
        :return:
        """
        # found, distance, point = None, 100, -1

        debug = False
        if debug:
            print 'SELECT'

        dmax = self._distance
        found_curve = None
        found_point = None

        point_dist = np.infty
        line_dist = np.infty
        note_dist = np.infty
        ellipse_dist = np.infty
        point_i = -1
        bellipse = False

        xmin = self.__plot.viewXMinSample()
        xmax = self.__plot.viewXMaxSample()
        rx = xmax - xmin
        # print xmin, xmax, (xmax - xmin)
        ymin = self.__plot.viewYMinSample()
        ymax = self.__plot.viewYMaxSample()
        ry = abs(ymax - ymin)
        # print ymin, ymax, (ymax - ymin)

        # normalize
        pos_clicked = pos
        pos = QPointF((pos.x() - xmin) / rx, (pos.y() - ymin) / ry)

        self.__unselect()

        d_ann = self.__plot.ann_get()
        for key, curve in d_ann.iteritems():

            t = curve.get_curve_type()

            if select_type is not None:
                if t != select_type:
                    continue

            x_from = (curve.x_from-xmin)/rx
            x_to = (curve.x_to-xmin)/rx
            y1 = y2 = None
            if curve.yval1 is not None:
                y1 = (curve.yval1-ymin)/ry

            if curve.yval2 is not None:
                y2 = (curve.yval2-ymin)/ry

            if t == EnumAnnType.basal:
                line_dist = abs(pos.y() - y1)
                if debug:
                    print 'basal (line dist):', line_dist

            elif t == EnumAnnType.baseline or t == EnumAnnType.recovery or t == EnumAnnType.no_recovery \
                    or t == EnumAnnType.excessive_ua:
                d0 = abs(x_from - pos.x())
                d1 = abs(x_to - pos.x())
                # print d0, d1

                if d0 < d1:
                    point_i = 0
                    point_dist = d0
                else:
                    point_i = 1
                    point_dist = d1

                line_dist = abs(pos.y() - y1)

                if debug:
                    print 'baseline etc (d0,d1,line dist):', d0, d1, line_dist

            elif t == EnumAnnType.ellipsenote:

                d = self.compute_dist_ellipse(x_from, x_to, y1, y2, pos)
                ellipse_dist = min(d)
                point_i = d.index(ellipse_dist)

                point_i = -1 if point_i == 0 else point_i

                # print d
                # print point_i, d[point_i]

                bx = x_from <= pos.x() <= x_to
                by = y1 <= pos.y() <= y2
                bellipse = bx and by
                ellipse_center = d[0]

                if debug:
                    print 'ellipse etc (ellipse_dist, bellipse):', ellipse_dist, bellipse

            elif t == EnumAnnType.note:
                note_dist = abs(pos.x() - x_from)
                if debug:
                    print 'note etc (note_dist):', note_dist
                # print note_dist

            else:
                return 0

            if point_dist < dmax and line_dist < dmax:
                # if a point is close
                found_curve = curve
                found_point = point_i
                # dmax_point = point_dist
                dmax = min(point_dist, line_dist)

            elif line_dist <= dmax and x_from < pos.x() < x_to:
                # if a line is close
                found_curve = curve
                found_point = -1
                dmax = line_dist

            elif note_dist < dmax:
                # if a note is close
                found_curve = curve
                found_point = -1
                dmax = note_dist

            elif ellipse_dist < dmax:
                found_curve = curve
                found_point = point_i
                dmax = ellipse_dist

            elif bellipse and found_curve is None:  # if not another curve has been selected
                found_curve = curve
                found_point = -1
                # dmax = ellipse_center

        if found_curve is not None:
            self.__selected_curve = found_curve

            if found_curve.get_curve_type() != EnumAnnType.ellipsenote:
                a = 5

            self.__selected_point = found_point
            self.__highlight(bhighlight)
            self.__point_clicked = pos_clicked
            return True

        return False

    def __select_curve(self, curve, point, pos):
        self.__selected_curve = curve
        self.__selected_point = point
        self.__point_clicked = pos

    def __unselect(self):
        self.__highlight(False)
        self.__selected_curve = None
        self.__selected_point = -1

    def __highlight(self, b_highlight):

        curve = self.__selected_curve
        # bdrawpoint = self.__selected_point != -1
        if not curve:
            return

        # Use copy constructors to defeat the reference semantics.
        # symbol = curve.get_symbol()
        new_symbol = curve.get_symbol()
        newpen = Qt.QPen(curve.get_default_pen())

        if b_highlight:
            new_symbol = curve.get_symbol_highlight()
            newpen.setColor(curve.get_color_highlight())

        doreplot = self.__plot.autoReplot()
        self.__plot.setAutoReplot(False)

        if curve.get_curve_type() != EnumAnnType.basal:
            curve.setSymbol(new_symbol)

        # curve.draw(self.__selected_point, self.__selected_point)
        # else:
        # if bdrawpoint:
        #     curve.setSymbol(symbol)
        # else:
        curve.setPen(newpen)
        # print curve.xAxis(), curve.yAxis()
        curve.draw_my(curve.xAxis(), curve.yAxis())

        self.__plot.setAutoReplot(doreplot)
        self.__plot.replot()

    def __delete(self):

        curve = self.__selected_curve
        if not curve:
            return

        self.__plot.ann_delete_curve(curve)

    def __move(self, pos, bfinished_move=False):
        """
        Move selected item

        :param pos: x and y position
        :param bfinished_move: if MouseRelease then moving was finished
        :type  pos: QPointF
        :type  bfinished_move: bool
        """

        curve = self.__selected_curve
        bdragpoint = self.__selected_point != -1

        if not curve:
            return

        # if isinstance(curve, PyQwtPlotMarkerAnnotator):
        #     pos = self._compute_point(pos)

        # x = self.__plot.invTransform(curve.xAxis(), pos.x())
        # y = self.__plot.invTransform(curve.yAxis(), pos.y())
        x = pos.x()
        y = pos.y()

        # if x < self.__plot.xAxisMinSample:
        #     x = self.__xAxisMinSample

        xmin = self.__plot.viewXMinSample()
        xmax = self.__plot.viewXMaxSample()

        ymin = self.__plot.viewYMinSample()
        ymax = self.__plot.viewYMaxSample()

        y = y if y > ymin else ymin
        y = y if y < ymax else ymax

        # if outside of box then move plot
        if x > xmax:
            self.signal_ann_moved_outside_view.emit(x-xmax)
        elif x < xmin:
            self.signal_ann_moved_outside_view.emit(x-xmin)

        # if a point is dragged
        if bdragpoint:

            if curve.get_curve_type() == EnumAnnType.ellipsenote:

                if self.__selected_point == 1:
                    curve.set_xy_values(curve.x_from, curve.x_to, curve.yval1, int(y))
                elif self.__selected_point == 2:
                    curve.set_xy_values(curve.x_from, int(x), curve.yval1, curve.yval2)
                elif self.__selected_point == 3:
                    curve.set_xy_values(curve.x_from, curve.x_to, int(y), curve.yval2)
                elif self.__selected_point == 4:
                    curve.set_xy_values(int(x), curve.x_to, curve.yval1, curve.yval2)
                elif self.__selected_point != 0:
                    curve.set_xy_values(curve.x_from, int(x), curve.yval1, int(y))
            else:
                # selected left of point of a curve
                if self.__selected_point == 0:
                    curve.set_xy_values(int(x), curve.x_to, curve.yval1, int(y))
                else:
                    # selected right of point of a curve
                    curve.set_xy_values(curve.x_from, int(x), curve.yval1, int(y))

            curve.plot()
            if bfinished_move:
                curve.correct_val_order()
                self.signal_ann_moved.emit()

        else:
            dx = int(x - self.__point_clicked.x())
            dy = int(y - self.__point_clicked.y())

            y1_new = y
            y2_new = y

            if curve.get_curve_type() != EnumAnnType.basal:
                x_new_from = curve.x_from + dx
                x_new_to = curve.x_to + dx
            else:
                x_new_from = curve.x_from
                x_new_to = curve.x_to

            if curve.get_curve_type() == EnumAnnType.ellipsenote:
                y1_new = curve.yval1 + dy
                y2_new = curve.yval2 + dy

            x_new_from, x_new_to, y1_new, y2_new = self.__check_outside_of_plot(x_new_from, x_new_to, y1_new, y2_new)
            curve.plot_xy(x_new_from, x_new_to, y1_new, y2_new)

            if bfinished_move:
                curve.set_xy_values(int(x_new_from), int(x_new_to), int(y1_new), int(y2_new))
                self.signal_ann_moved.emit()

        self.__plot.replot()

    # def event(self, event):
    #     if event.type() == Qt.QEvent.User:
    #         return True
    #     return Qt.QObject.event(event)

    def eventFilter(self, obj, event):
        """
        Filter mouse and key events
        """

        if event.type() == Qt.QEvent.Paint:
            Qt.QApplication.postEvent(self, Qt.QEvent(Qt.QEvent.User))

        elif event.type() == Qt.QEvent.MouseButtonPress:  # and self.__bleft_btn_hold is False:
            self.__last_click_pos = event.pos()
            self.perform_single_click_event(event)

            # Qt.QTimer.singleShot(Qt.QApplication.instance().doubleClickInterval(),
            #                      self.perform_single_click_event)

        elif event.type() == Qt.QEvent.MouseButtonDblClick:
            self.perform_double_click_event(event)

        elif event.type() == Qt.QEvent.MouseMove:
            point = self.__compute_point(event.pos())
            self.__bmoving = True
            self.__move(point)
            return True

        elif event.type() == Qt.QEvent.MouseButtonRelease:
            if self.__bmoving:
                point = self.__compute_point(event.pos())
                self.__bmoving = False
                self.__move(point, True)

                if self.__addannotation is True and not self.__selected_curve.is_too_small():
                    self.__addannotation = False
                    self._add_note_dialog.clear_text()
                    if self._add_note_dialog.show():
                        # self._current_curve.set_text(self._add_note_dialog.get_text())
                        self.__selected_curve.set_text(self._add_note_dialog.get_text())
                        self.__plot.ann_plot_curves()

            if self.__selected_curve is not None:
                if self.__selected_curve.is_too_small():
                    self.__delete()

        elif event.type() == Qt.QEvent.KeyPress:
            if event.key() == Qt.Qt.Key_Delete:
                self.__delete()

            # elif event.matches(Qt.QKeySequence.Copy):
            #     if self.__selected_curve is not None:
            #         self.__curve_to_copy = self.__selected_curve.copy()
            #
            # elif event.matches(Qt.QKeySequence.Cut):
            #     if self.__selected_curve is not None:
            #         self.__curve_to_copy = self.__selected_curve.copy()
            #         self.__delete()
            #
            # elif event.matches(Qt.QKeySequence.Paste):
            #
            #     if self.__curve_to_copy is not None:
            #         print self.__last_click_pos
            #         print self.__curve_to_copy
            #         d = int(self.__curve_to_copy.x_to - self.__curve_to_copy.x_to)
            #         self.__curve_to_copy.x_from = int(self.__last_click_pos.x())
            #         self.__curve_to_copy.x_to = self.__curve_to_copy.x_from + d
            #         self.__curve_to_copy.yval1 = int(self.__last_click_pos.y())
            #         self.__plot.ann_add(self.__curve_to_copy)
            #         # self.__curve_to_copy = None

        return Qwt.QwtPlot.eventFilter(self, obj, event)

    def perform_single_click_event(self, event):
        """
        Perform event related to a single click on plot
        :param event:
        :type event:
        """

        action = self.__plot.get_ann_action()
        point = self.__compute_point(event.pos())

        if event.button() == Qt.Qt.LeftButton:

            self.__unselect()

            if action == EnumAnnType.basal:
                self.__plot.ann_basal(point.y())

            elif action == EnumAnnType.baseline or action == EnumAnnType.recovery or \
                            action == EnumAnnType.no_recovery or action == EnumAnnType.excessive_ua:

                curve = self.__plot.ann_line_start(point, action)
                self.__select_curve(curve, point, point)

            elif action == EnumAnnType.ellipsenote:
                self._add_note_dialog.clear_text()

                if not (self.__addannotation is False and self.__select(point, True, EnumAnnType.ellipsenote)):
                    curve = self.__plot.ann_ellipse(point, action)
                    self.__select_curve(curve, point, point)
                    self.__addannotation = True
                else:
                    self.__unselect()

            elif action == EnumAnnType.note:
                self._add_note_dialog.clear_text()
                if not self.__select(point, True, EnumAnnType.note):  # add new annotation
                    if self._add_note_dialog.show():
                        self.__plot.ann_note(point, self._add_note_dialog.get_text())
                else:
                    self.__unselect()

            elif action == EnumAnnType.select:
                self.__select(point)

    def perform_double_click_event(self, event):
        """
        Perform event related to a double click (edit the annotations)
        """

        action = self.__plot.get_ann_action()
        point = self.__compute_point(event.pos())

        if event.button() == Qt.Qt.LeftButton:
            if action == EnumAnnType.select:
                self._add_note_dialog.clear_text()

                if (self.__addannotation is False and self.__select(point, True, EnumAnnType.ellipsenote)) \
                        or (self.__select(point, True, EnumAnnType.note)):
                    # modify current annotation
                    self._add_note_dialog.set_text(self.__selected_curve.text.text())
                    if self._add_note_dialog.show():
                        self.__selected_curve.set_text(self._add_note_dialog.get_text())
                        self.signal_ann_moved.emit()
                        self.__plot.ann_plot_curves()

# def main():
#     app = Qt.QApplication(sys.argv)
#     window = AnnotatorTemp()
#     window.resize(680, 400)
#     window.show()
#     sys.exit(app.exec_())
#
# if __name__ == '__main__':
#     main()
