# -*- coding: utf-8 -*-
#
# Created on Oct 15, 2013
# @author: Jiri Spilka
# http://people.ciirc.cvut.cz/~spilkjir
# @2015, CIIRC, Czech Technical University in Prague
#
# Licensed under the terms of the GNU GENERAL PUBLIC LICENSE
# (see CTGViewerLite.py for details)

"""
MainWindow
----------------

The MainWindow module is the main class that provides interaction between GUI and the user.
This class integrates all the other functionality.

Reference
~~~~~~~~~~~
.. autoclass:: Main
    :members:
    :private-members:
"""

# system imports
import argparse
import logging
import os
import sys
from PyQt4 import QtCore, QtGui
# local imports
from MainWindowUI import Ui_MainWindow
from GuiForms import AboutDialog, ConvertFileForm, DownloadDbForm, SentAnnotations
from Config import ConfigStatic, EnumIniVar, ConfigIni
from LoadWriteData import LoadData
from LoadWriteData import EnumVariableName as EnumVarName
from ClinInfoForm import ClinInfo
from DataBrowserForm import DataBrowserForm
from DataBrowserSelectAttrForm import DataBrowserSelectAttrForm
from Init import init
from AnnotationObject import EnumAnnType
import Common

DEBUG_PROFILE = False

# conditional import
if DEBUG_PROFILE:
    import cProfile
    import pstats


class Main(QtGui.QMainWindow):

    def __init__(self, args=sys.argv):
        QtGui.QMainWindow.__init__(self)

        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        self._winTitleDefault = str(self.windowTitle())

        self._log = logging.getLogger(ConfigStatic.logger_name)
        self._log.info('passed')
        self._log.info('profiling: {0}'.format(DEBUG_PROFILE))

        self._dataLoader = LoadData()

        self._config_ini = ConfigIni()
        self._convertFilesForm = ConvertFileForm()
        self._downloadDbForm = DownloadDbForm()
        self._attSelectForm = DataBrowserSelectAttrForm(self._config_ini)
        self._sent_ann_form = SentAnnotations(self._config_ini)

        # dock widgets
        self._clinInfoWidget = ClinInfo()
        self._clinInfoWidget.clear_all()
        self.ui.dockClinInfo.setWidget(self._clinInfoWidget)
        # self.ui.dockClinInfo.setVisible(self._config_ini.get_var(EnumIniVar.dockClinInfoVisible))

        self._dataBrowserWidget = DataBrowserForm()
        self.ui.dockDataBrowser.setWidget(self._dataBrowserWidget)
        # self.ui.dockDataBrowser.setVisible(self._config_ini.get_var(EnumIniVar.dockDataBrowseVisible))
        self._dataBrowserWidget.plotFileSignal.connect(self.plot_file)
        self._dataBrowserWidget.debug_stageI_signal.connect(self.debug_plot_stage1)

        # val = self._config_ini.get_var(EnumIniVar.annotationToolbar)
        valr = self._config_ini.get_var(EnumIniVar.annotationToolbarAlignR)
        self.ui.actionAnnToolbarAlign_right.setChecked(valr)
        self._toolbar_align()

        self.ui.actionSent_annotations.setEnabled(False)

        self._create_connections()

        # self.ui.dockClinInfo.dockWidgetArea
        self._debug_data = dict()

        self.msgbox_unsaved = QtGui.QMessageBox()
        self.msgbox_unsaved.setText("There are unsaved changes in annotations.")
        self.msgbox_unsaved.setInformativeText("Save now?")
        self.msgbox_unsaved.setStandardButtons(QtGui.QMessageBox.Save | QtGui.QMessageBox.Discard)
        self.msgbox_unsaved.setDefaultButton(QtGui.QMessageBox.Save)
        self.msgbox_unsaved.setIcon(QtGui.QMessageBox.Critical)

        self.msgbox_delete = QtGui.QMessageBox()
        self.msgbox_delete.setText("Are you sure to delele all annotations?")
        self.msgbox_delete.setInformativeText("All annotations will be lost. This can't be undone.")
        self.msgbox_delete.setStandardButtons(QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        self.msgbox_delete.setDefaultButton(QtGui.QMessageBox.Save)
        self.msgbox_delete.setIcon(QtGui.QMessageBox.Warning)

        self._process_cmd_args(args)

        # paper format
        self._set_paper_format(self._config_ini.get_var(EnumIniVar.paperformat))

        # self._save_to_pdf2()
        valg = QtCore.QByteArray(self._config_ini.get_var(EnumIniVar.windowGeometry))
        vals = QtCore.QByteArray(self._config_ini.get_var(EnumIniVar.windowState))
        self.restoreGeometry(valg)
        self.restoreState(vals)

        if len(vals) == 0:
            self.ui.dockDataBrowser.setVisible(False)
            self.ui.dockClinInfo.setVisible(False)
            self.ui.toolBar.setVisible(False)

        self._dock_clin_info_visibility()
        self._dock_databrowse_visibility()
        self._toolbar_ann_visibility()

    def _create_connections(self):
        """ Creates connections in MainWindow """

        # Menu: File
        self.connect(self.ui.actionQuit, QtCore.SIGNAL('triggered()'), QtCore.SLOT('close()'))
        self.connect(self.ui.actionOpen, QtCore.SIGNAL('triggered()'), self._open_file)
        self.connect(self.ui.actionOpen_folder, QtCore.SIGNAL('triggered()'), self._open_folder_dialog)

        # Menu: TOOLS
        self.connect(self.ui.actionConvert_files, QtCore.SIGNAL('triggered()'), self._show_convert_files)
        self.connect(self.ui.actionDownload_CTU_UHB_data, QtCore.SIGNAL('triggered()'), self._show_download_db)
        self.connect(self.ui.actionAttribute_selection, QtCore.SIGNAL('triggered()'), self._show_attribute_selection)
        self.connect(self.ui.actionExport_to_PDF, QtCore.SIGNAL('triggered()'), self._save_to_pdf)
        # self.connect(self.ui.actionSet_Clear_Baseline, QtCore.SIGNAL('triggered()'), self._show_set_clear_bsln)
        self.connect(self.ui.actionSent_annotations, QtCore.SIGNAL('triggered()'), self._show_sent_annotations)

        group_paper = QtGui.QActionGroup(self)
        self.ui.actionEU.setActionGroup(group_paper)
        self.ui.actionUS.setActionGroup(group_paper)
        self.connect(self.ui.actionEU, QtCore.SIGNAL('triggered()'), self._set_paper)
        self.connect(self.ui.actionUS, QtCore.SIGNAL('triggered()'), self._set_paper)

        group_ann = QtGui.QActionGroup(self)
        self.ui.actionCursor.setActionGroup(group_ann)
        self.ui.actionBasal.setActionGroup(group_ann)
        self.ui.actionBaseline.setActionGroup(group_ann)
        self.ui.actionRecovery.setActionGroup(group_ann)
        self.ui.actionNo_recovery.setActionGroup(group_ann)
        self.ui.actionExcessive_UA.setActionGroup(group_ann)
        self.ui.actionEllipse.setActionGroup(group_ann)
        self.ui.actionEllipseNote.setActionGroup(group_ann)
        self.ui.actionNote.setActionGroup(group_ann)
        self.connect(self.ui.actionCursor, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionBasal, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionBaseline, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionRecovery, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionNo_recovery, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionExcessive_UA, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionEllipse, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionEllipseNote, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionNote, QtCore.SIGNAL('triggered()'), self._set_annotation_action)
        self.connect(self.ui.actionSave, QtCore.SIGNAL('triggered()'), self._toolbar_ann_save)
        self.connect(self.ui.actionDelete, QtCore.SIGNAL('triggered()'), self._toolbar_ann_delete)

        # Menu: View
        self.connect(self.ui.actionClinical_information, QtCore.SIGNAL('triggered()'), self._dock_clin_info_toggle)
        self.connect(self.ui.dockClinInfo, QtCore.SIGNAL("visibilityChanged(bool)"), self._dock_clin_info_visibility)
        self.connect(self.ui.actionData_browser, QtCore.SIGNAL('triggered()'), self._dock_databrowse_toggle)
        self.connect(self.ui.dockDataBrowser, QtCore.SIGNAL("visibilityChanged(bool)"),
                     self._dock_databrowse_visibility)
        self.connect(self.ui.actionAnnToolbarVisible, QtCore.SIGNAL('triggered()'), self._toolbar_ann_toogle)
        self.connect(self.ui.toolBar, QtCore.SIGNAL('visibilityChanged(bool)'), self._toolbar_ann_visibility)
        self.connect(self.ui.actionAnnToolbarAlign_right, QtCore.SIGNAL('triggered()'), self._toolbar_align)

        # Menu: Help
        self.connect(self.ui.actionAbout, QtCore.SIGNAL('triggered()'), self._show_about)

        # self.connect(self.ui.actionCursor, QtCore.SIGNAL('triggered()'), self._save_to_pdf2)

        # SIGNALS
        self.ui.PlotWidget.fhrPlot.signal_ann_changed.connect(self._toolbar_ann_changed)
        self.ui.PlotWidget.tocoPlot.signal_ann_changed.connect(self._toolbar_ann_changed)
        self._attSelectForm.signal_sel_att_changed.connect(self._update_data_browser)

    def _get_paper_format(self):
        """

        :return: paper format
        :rtype: str
        """
        if self.ui.actionEU.isChecked():
            return 'EU'
        else:
            return 'US'

    def _dock_clin_info_visibility(self):
        """ Clinical information dock visibility has changed, set the property of checkbox"""
        self.ui.actionClinical_information.setChecked(self.ui.dockClinInfo.isVisible())

    def _dock_clin_info_toggle(self):
        """ Set clinical information dock widget visible or invisible  """
        if self.ui.actionClinical_information.isChecked():
            self.ui.dockClinInfo.show()
        else:
            self.ui.dockClinInfo.hide()

    def _dock_databrowse_visibility(self):
        """ Visibility of data browser has changed, set the property of checkbox. """
        self.ui.actionData_browser.setChecked(self.ui.dockDataBrowser.isVisible())

    def _dock_databrowse_toggle(self):
        """ Set docks widget visible or invisible  """
        if self.ui.actionData_browser.isChecked():
            self.ui.dockDataBrowser.show()
        else:
            self.ui.dockDataBrowser.hide()

    def _toolbar_ann_toogle(self):
        """ Set toolbar visible  or invisible  """
        if self.ui.actionAnnToolbarVisible.isChecked():
            self.ui.toolBar.setVisible(True)
        else:
            self.ui.toolBar.setVisible(False)

    def _toolbar_ann_visibility(self):
        self.ui.actionAnnToolbarVisible.setChecked(self.ui.toolBar.isVisible())

    def _toolbar_ann_changed(self):
        """ Annotations were changed. Enable saving button. """
        self.ui.actionSave.setEnabled(True)

    def _toolbar_ann_save(self):
        # print self.ui.PlotWidget.fhrPlot.__d_ann_all_curves
        # print len(self.ui.PlotWidget.fhrPlot.__d_ann_all_curves)
        self.ui.PlotWidget.ann_save()
        # self._annotator.set_annotations_and_save(self.ui.PlotWidget.fhrPlot.__d_ann_all_curves)
        self.ui.actionSave.setEnabled(False)

    def toolbar_restore_geometry(self):

        # self.ui.toolBar.restoreGeometry(self._valg)
        self.restoreGeometry(self._valg)

    def _toolbar_ann_delete(self):

        # print self.ui.toolBar.saveGeometry()
        ret = self.msgbox_delete.exec_()

        if ret == QtGui.QMessageBox.Yes:
            self.ui.PlotWidget.delete_annotations()

    def _toolbar_align(self):

        self.ui.toolBar.clear()

        # self.ui.actionEllipseNote.setEnabled(True)
        # self.ui.actionEllipseNote.setCheckable(True)

        # tmenu = QtGui.QMenu()
        # tmenu.addAction(self.ui.actionEllipse)
        # tmenu.addAction(self.ui.actionEllipseNote)
        # toolb = QtGui.QToolButton()
        # toolb.setMenu(tmenu)
        # toolb.setDefaultAction(self.ui.actionEllipseNote)
        # toolb.setPopupMode(QtGui.QToolButton.MenuButtonPopup)

        if self.ui.actionAnnToolbarAlign_right.isChecked():
            spacer = QtGui.QWidget()
            spacer.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
            self.ui.toolBar.addWidget(spacer)

        self.ui.toolBar.addAction(self.ui.actionCursor)
        self.ui.toolBar.addAction(self.ui.actionBasal)
        self.ui.toolBar.addAction(self.ui.actionBaseline)
        self.ui.toolBar.addAction(self.ui.actionRecovery)
        self.ui.toolBar.addAction(self.ui.actionNo_recovery)
        self.ui.toolBar.addAction(self.ui.actionExcessive_UA)
        # self.ui.toolBar.addWidget(toolb)
        # self.ui.toolBar.addAction(self.ui.actionEllipse)
        self.ui.toolBar.addAction(self.ui.actionEllipseNote)
        self.ui.toolBar.addAction(self.ui.actionNote)
        self.ui.toolBar.addSeparator()
        self.ui.toolBar.addAction(self.ui.actionSave)
        self.ui.toolBar.addAction(self.ui.actionDelete)

    def _ann_check_unsaved_changes(self):

        if self.ui.actionSave.isEnabled():
            ret = self.msgbox_unsaved.exec_()

            if ret == QtGui.QMessageBox.Save:
                self._toolbar_ann_save()
            else:
                self.ui.PlotWidget.delete_annotations()
                self.ui.actionSave.setEnabled(False)

    def _open_file(self):
        """ Open file in gui. Files are filtered based on extension """

        self._ann_check_unsaved_changes()

        val = self._config_ini.get_var(EnumIniVar.lastUsedDirFiles)
        file1 = QtGui.QFileDialog.getOpenFileName(self, 'Open File', val, 'Files (*.dat *.csv *.mat *.txt)')

        if file1 == "":
            self._log.debug("File name is empty - no file selected")
            return -1

        file1 = str(file1)
        dummy, ext = Common.get_filename_and_ext(file1)

        if ext == '.dat' or '.mat':
            self.plot_file(file1)

        # if ext == '.dat':
        #     self.plot_physionet_file(file1)
        # elif ext == '.mat':
        #     self.plot_matlab_file(file1)
        if ext == '.csv':
            self.plot_csv_file(file1)
        elif ext == '.txt':
            self.plot_bditxt_file(file1)

        self._config_ini.set_var(EnumIniVar.lastUsedDirFiles, os.path.dirname(file1))
        return 0

    def _open_folder_data_browser(self, sdir):
        """ Open folder in gui.  """

        self._ann_check_unsaved_changes()
        self.ui.dockDataBrowser.show()
        self.ui.actionData_browser.setChecked(True)

        if not os.path.exists(sdir):
            self._log.error('Chosen directory do not exists: {0}'.format(sdir))

        self._dataBrowserWidget.set_attributes_dir_and_load(self._attSelectForm.get_elected_att(), sdir)
        self._config_ini.set_var(EnumIniVar.lastUsedDirFiles, sdir)
        return 0

    def _open_folder_dialog(self):

        sdir = QtGui.QFileDialog.getExistingDirectory(self, 'Open Folder', self._config_ini.get_var(EnumIniVar.lastUsedDirFiles))
        if sdir == "":
            self._log.debug("Data browser open directory: Name is empty - none selected")
            return -1

        sdir = str(sdir)
        self._open_folder_data_browser(sdir)

    def debug_plot_stage1(self, pos_stage1):

        d = self._debug_data
        if len(d) == 0:
            return

        stage1_min = pos_stage1
        sig2end_min = d['Sig2Birth']
        fs = d['fs']
        n = len(d['fhr'])

        import numpy as np
        if stage1_min <= sig2end_min:
            pos_stage1 = np.nan
        else:
            pos_stage1 = n - round(60 * fs * (stage1_min - sig2end_min))

        # print pos_stage1
        self.ui.PlotWidget.plot_stage1_line(pos_stage1)
        self.ui.PlotWidget.updatePlots()

    def plot_file(self, sfile):
        """
        Plot physionet or matlab file

        :param sfile: input file
        :type sfile: str
        """
        sfile = str(sfile)

        self._ann_check_unsaved_changes()

        try:
            adata = self._dataLoader.read_data(sfile)
        except Exception as ex:
            self._log.error(ex)
            return

        # self._annotator.set_and_load_ann_file(sfile)

        afhr = adata[EnumVarName.fhr]
        auc = adata[EnumVarName.uc]
        atimestamp = adata[EnumVarName.timestamp]
        fs = adata[EnumVarName.fs]
        nr_samples = len(afhr)

        time_string = self.ui.PlotWidget.fhrPlot.samples2time(nr_samples, fs)
        self.ui.PlotWidget.setSamplingFreqAllPlots(fs)
        self.ui.PlotWidget.plot(atimestamp, afhr, auc, time_string)

        # if self.ui.actionAnnotation_Toolbar.isChecked():
        self.ui.PlotWidget.ann_file_load_and_plot(sfile)
        # self.ui.actionCursor.setChecked(True)
        # self.ui.PlotWidget.navPlot.plot_ann_navplot_marker()

        self._debug_data = adata

        if 'Pos_IIst' in adata:
            val = adata['Pos_IIst']
            if val != -1:
                self.ui.PlotWidget.plot_stage2_line(adata['Pos_IIst'])
                self.ui.PlotWidget.updatePlots()

        if 'name' in adata:
            self._set_window_title_rec(adata['name'])

        # clinical information
        self._clinInfoWidget.set_all(adata)

    def plot_physionet_file(self, sfile):
        """ Obsolete function - use plot file instead

        :param sfile:
        :return:
        """
        sfile = str(sfile)

        try:
            adata, lheader = self._dataLoader.read_physionet_signal16(sfile)
        except Exception as ex:
            self._log.error(ex)
            return

        sig = lheader[0]
        dictclininfo = lheader[2]
        nfs = sig['fs']
        self.ui.PlotWidget.setSamplingFreqAllPlots(nfs)

        nnrsamples = len(adata[EnumVarName.time_samp])
        timestring = self.ui.PlotWidget.fhrPlot.samples2time(nnrsamples, nfs)
        self.ui.PlotWidget.plot(adata[EnumVarName.time_samp], adata[EnumVarName.fhr], 
                                adata[EnumVarName.uc], timestring)

        if 'Pos_IIst' in dictclininfo:
            # print dictclininfo['Pos_IIst']
            self.ui.PlotWidget.plot_stage2_line(dictclininfo['Pos_IIst'])
            self.ui.PlotWidget.updatePlots()

        if 'name' in sig:
            self._set_window_title_rec(sig['name'])
        # clinical information
        self._clinInfoWidget.set_all(dictclininfo)

    def plot_matlab_file(self, sfile):
        """ Obsolete function - use plot file instead

        :param sfile:
        :return:
        """
        try:
            adata = self._dataLoader.read_matlab_file(sfile)
        except Exception as ex:
            self._log.error(ex)
            return

        # nFs = adata[EnumVarName.fs]
        afhr = adata[EnumVarName.fhr]
        auc = adata[EnumVarName.uc]
        # aTime_samp = adata[EnumVarName.time_samp]
        atimestamp = adata[EnumVarName.timestamp]

        fs = adata[EnumVarName.fs]
        self.ui.PlotWidget.setSamplingFreqAllPlots(fs)

        # self._annotator.set_and_load_ann_file(sfile)

        nr_samples = len(afhr)
        time_string = self.ui.PlotWidget.fhrPlot.samples2time(nr_samples, fs)
        self.ui.PlotWidget.plot(atimestamp, afhr, auc, time_string)

        if 'Pos_IIst' in adata:
            # print adata['Pos_IIst']
            self.ui.PlotWidget.plot_stage2_line(adata['Pos_IIst'])
            self.ui.PlotWidget.updatePlots()

        if 'name' in adata:
            self._set_window_title_rec(adata['name'])

        # clinical information
        self._clinInfoWidget.set_all(adata)

    def plot_csv_file(self, sfile):

        try:
            adata, dummy = self._dataLoader.read_csv_file(sfile)
        except Exception as ex:
            self._log.error(ex)
            return

        afhr = adata[EnumVarName.fhr]
        uc = adata[EnumVarName.uc]
        atimestamp = adata[EnumVarName.timestamp]

        self.ui.PlotWidget.plot(atimestamp, afhr, uc)

    def plot_bditxt_file(self, sfile):

        try:
            adata, dummy = self._dataLoader.read_bdi_txtfile(sfile)
        except Exception as ex:
            self._log.error(ex)
            return

        afhr = adata[EnumVarName.fhr]
        uc = adata[EnumVarName.uc]
        atimestamp = adata[EnumVarName.timestamp]

        self.ui.PlotWidget.plot(atimestamp, afhr, uc)

    def _show_about(self):
        win = AboutDialog()
        win.exec_()

    def _show_convert_files(self):
        self._convertFilesForm.show()

    def _show_attribute_selection(self):
        self._attSelectForm.show()

        # msg = QtGui.QMessageBox()
        # msg.setText('Not implemented yet.')
        # msg.setIcon(QtGui.QMessageBox.Warning)
        # msg.setStandardButtons(QtGui.QMessageBox.Ok)
        # msg.exec_()

    def _show_download_db(self):
        self._downloadDbForm.show()

    def _show_sent_annotations(self):
        self._sent_ann_form.clear_and_show()

    # def _show_set_clear_bsln(self):
    #     self._setclearbsln.show()

    # def _update_baseline_constant(self, val):
    #     print val
    #     self.ui.PlotWidget.plot_baseline_constant(val)
    #     self.ui.PlotWidget.updatePlots()

    def _set_paper_format(self, val):
        if val == 'EU':
            self.ui.actionEU.setChecked(True)
        else:
            self.ui.actionUS.setChecked(True)

        self._set_paper()

    def _set_paper(self):
        if self.ui.actionEU.isChecked():
            self.ui.PlotWidget.set_paper_eu()
        else:
            self.ui.PlotWidget.set_paper_us()

    def _set_annotation_action(self):

        p_fun = self.ui.PlotWidget.set_ann_action

        if self.ui.actionCursor.isChecked():
            p_fun(EnumAnnType.select)

        elif self.ui.actionBasal.isChecked():
            p_fun(EnumAnnType.basal)

        elif self.ui.actionBaseline.isChecked():
            p_fun(EnumAnnType.baseline)

        elif self.ui.actionRecovery.isChecked():
            p_fun(EnumAnnType.recovery)

        elif self.ui.actionNo_recovery.isChecked():
            p_fun(EnumAnnType.no_recovery)

        elif self.ui.actionExcessive_UA.isChecked():
            p_fun(EnumAnnType.excessive_ua)

        # elif self.ui.actionEllipse.isChecked():
        #     p_fun(EnumAnnType.ellipse)

        elif self.ui.actionEllipseNote.isChecked():
            p_fun(EnumAnnType.ellipsenote)

        elif self.ui.actionNote.isChecked():
            p_fun(EnumAnnType.note)
        else:
            self._log.debug("_set_annotation_action: unknow action")

    def _set_window_title_rec(self, name=None):

        if name is None:
            self.setWindowTitle(self._winTitleDefault)
        else:
            s = self._winTitleDefault + ' (' + name + ')'
            self.setWindowTitle(s)

    def _save_to_pdf(self):
        filename = QtGui.QFileDialog.getSaveFileName(self,'Export File Name', '%s.pdf' % self.windowTitle(),
                                                     'PDF Documents (*.pdf)')
        if not filename.isEmpty():
            printer = QtGui.QPrinter()
            printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
            printer.setOrientation(QtGui.QPrinter.Landscape)
            printer.setOutputFileName(filename)

            printer.setCreator('CTGViewerLite')
            self.ui.PlotWidget.fhrPlot.print_(printer)

    # def _save_to_pdf2(self):
    #
    #     filename = '/home/jirka/svn_working_copy/iga_brno/python/ctgViewerLite/src/test.pdf'
    #     printer = QtGui.QPrinter()
    #
    #     printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
    #     printer.setOrientation(QtGui.QPrinter.Landscape)
    #     printer.setOutputFileName(filename)
    #     printer.setCreator('CTGViewerLite')
    #
    #     paint = QtGui.QPainter(printer)
    #
    #     # self.ui.PlotWidget.fhrPlot.print_(printer, QtCore.QRect(0, 0, 10, 10))
    #     self.ui.PlotWidget.fhrPlot.print_(paint, QtCore.QRect(0, 0, 10, 10))

    def _update_data_browser(self):
        self._dataBrowserWidget.update_model(self._attSelectForm.get_elected_att())

    def closeEvent(self, event):

        self._ann_check_unsaved_changes()
        # self._config_ini.set_var(EnumIniVar.dockClinInfoVisible, self.ui.dockClinInfo.isVisible())
        # self._config_ini.set_var(EnumIniVar.dockDataBrowseVisible, self.ui.dockDataBrowser.isVisible())
        # self._config_ini.set_var(EnumIniVar.annotationToolbar, self.ui.toolBar.isVisible())
        self._config_ini.set_var(EnumIniVar.annotationToolbarAlignR, self.ui.actionAnnToolbarAlign_right.isChecked())
        self._config_ini.set_var(EnumIniVar.paperformat, self._get_paper_format())
        self._config_ini.set_var(EnumIniVar.windowGeometry, self.saveGeometry())
        self._config_ini.set_var(EnumIniVar.windowState, self.saveState())
        self._config_ini.write_config()

    def _process_cmd_args(self, args):

        parsed_args = parse_cmd_args()

        if parsed_args.physionet_file is not None:
            self.plot_file(parsed_args.physionet_file)
            # self.plot_physionet_file(parsed_args.physionet_file)

        if parsed_args.matlab_file is not None:
            self.plot_file(parsed_args.matlab_file)
            # self.plot_matlab_file(parsed_args.matlab_file)

        if parsed_args.folder is not None:
            self._open_folder_data_browser(parsed_args.folder)

        # print parsed_args.physionet_file
        # print parsed_args.matlab_file


def parse_cmd_args():
    parser = argparse.ArgumentParser(description='ctgViewer -- browsing and viewing CTU-UHB database')
    parser.add_argument('-p', '--physionet-file', help='Input file in the physionet format', required=False)
    parser.add_argument('-m', '--matlab-file', help='Input file in the matlab format', required=False)
    parser.add_argument('-f', '--folder', help='Input folder for browsing', required=False)
    return parser.parse_args()


def main():

    init()
    app = QtGui.QApplication(sys.argv)
    window = Main(sys.argv)
    window.show()
    window.ui.PlotWidget.updatePlots()

    sys.exit(app.exec_())

if __name__ == '__main__':

    if DEBUG_PROFILE:
        cProfile.run('main()', 'profile_data')
        p = pstats.Stats('profile_data')
        p.sort_stats('time').print_stats(20)
        p.sort_stats('cumulative').print_stats(20)
    else:
        main()
